<?php
/**
 * @package   BitNinja SiteProtection for WP
 * @version   1.0.1
 * @author    Viktor Kiss
 * @copyright 2020 BitNinja SiteProtection
 *
 * Text Domain: bitninja_domain
 * Domain Path: /lang
 * Plugin Name: BitNinja SiteProtection WordPress Admin
 * Plugin URI: http://siteprotection.io/
 * Description: BitNinja SiteProtection extension for WordPress
 * Author: Viktor Kiss
 * Version: 1.0.1
 * Author URI: http://siteprotection.io/
 */
if ( ! defined( 'ABSPATH' ) ) exit; // Prevent direct access

/**
 * Main Genius Pizza Batch Editor Class.
 *
 * @class WP_BitNinja_Admin
 */
class WP_BitNinja_Admin {

	/**
	 * The single instance of the class.
	 */
	static $instance = false;

	static $plugin_capability = 'manage_options';

	static $agent_url = 'http://localhost:60222';

	static $sp_url = 'https://app.siteprotection.io/';

	static $login_message = false;

	/**
	 * Main WP_BitNinja_Admin Instance.
	 *
	 * Ensures only one instance of WP_BitNinja_Admin is loaded or can be loaded.
	 * @return WP_BitNinja_Admin - Main instance.
	 */
	public static function getInstance() {
		if ( !self::$instance ) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	/**
	 * Class Constructor.
	 */
	private function __construct() {
		$this->define_constants();
		$this->includes();
		$this->init_hooks();

		do_action( 'bitninja_plugin_loaded' );
	}

	/**
	 * Define GP Constants.
	 */
	private function define_constants() {
		$this->define( 'BITNINJA_PLUGIN_URL', plugin_dir_url( __FILE__ ) );
		$this->define( 'BITNINJA_PLUGIN_DIR', plugin_dir_path( __FILE__ ) );
		$this->define( 'N', "\n" );
	}

	/**
	 * Define constant if not already set.
	 *
	 * @param  string $name
	 * @param  string|bool $value
	 */
	private function define( $name, $value ) {
		if ( ! defined( $name ) ) {
			define( $name, $value );
		}
	}

	/**
	 * includes for classes
	 */
	private function includes() {
	}

	/**
	 * Hook into actions and filters.
	 */
	private function init_hooks() {
		add_action( 'admin_menu', array( $this, 'admin_menu' ), 10 );
		add_action( 'admin_notices', array( $this, 'admin_notice' ) );
		add_action( 'admin_init', array( $this, 'check_login' ), 100 );
	}

	public function check_login() {
		global $pagenow;
		if ( $pagenow == 'admin.php' && isset( $_GET['page'] ) && $_GET['page'] == 'bitninja_admin' && isset( $_GET['login'] ) && $_GET['login'] == 1 ) {
			self::$login_message = false;

			$filename = 'valid_' . $this->random_string( 12 ) . '.txt';
			$filepath = BITNINJA_PLUGIN_DIR . 'login/' . $filename;
			$validation = $this->random_string( 32 );
			$success = file_put_contents( $filepath, $validation );
			$data = array(
				'extensionFamily' => 'wordpress',
				'extensionVersion' => '1.0.0',
				'path' => $filepath,
				'emailAddress' => get_option('admin_email')
			);
			
			if ( $success ) {
				$i = 0;
				do {
					$i++;
					$noToken = false;
					$response = $this->getToken($data);
					$tokens = json_decode($response);
					if ((empty($tokens->accessToken) && empty($tokens->refreshToken)) && $i <= 10) {
						usleep(25000); //0.25 sec
						$noToken = true;
					}
				} while ($noToken);
				$siteProtectionUrl = false;
				if (isset($tokens->accessToken) && isset($tokens->refreshToken)) {
					$siteProtectionUrl = self::$sp_url . "auth?accessToken=" . $tokens->accessToken . "&refreshToken=" . $tokens->refreshToken;
				}

				if ($siteProtectionUrl) {
					if ( wp_redirect( $siteProtectionUrl ) ) {
						die;
					}
					else {
						self::$login_message =  __( 'Redirection error!', 'bitninja_domain' ) . N;
					}
				}
				else {
					self::$login_message =  __( 'Token error!', 'bitninja_domain' ) . N;
				}
			}
			else {
				self::$login_message =  __( 'Validation file create error', 'bitninja_domain' ) . N;
			}
		}
	}

	public function admin_notice() {
		$ack = get_option( 'bitninja_helper_ack', 0 );
		if ( $ack ) {
			return true;
		}
		if ( !current_user_can( self::$plugin_capability ) ) {
			return true;
		}
		global $pagenow;
		if ( $pagenow != 'index.php' ) {
			return true;
		}
		echo '<div class="notice notice-info">' . N;
		echo __( 'Great news! Your host is protected by BitNinja SiteProtection! Do you want to know more of this?<br>Just check the SiteProtection in your admin menu!', 'bitninja_domain' ) . N;
		echo '</div>' . N;
	}

	public function admin_menu() {
		add_menu_page( __( 'SiteProtection', 'bitninja_domain' ),  __( 'SiteProtection', 'bitninja_domain' ) , self::$plugin_capability, 'bitninja_admin', array( $this, 'admin_content' ), 'dashicons-welcome-widgets-menus', '10' );
	}

	private function random_string( $length = 32 ) {
		$chrList = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
		return substr( str_shuffle( str_repeat( $chrList, mt_rand( 5,10 ) ) ), 1, $length );
	}

	private function getToken( $params ) {
		$ch = curl_init();
		$data_string = json_encode( $params );
		curl_setopt( $ch, CURLOPT_URL, self::$agent_url );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1 );
		curl_setopt( $ch, CURLOPT_POST, true );
		curl_setopt( $ch, CURLOPT_POSTFIELDS, $data_string );
		curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 1 );
		curl_setopt( $ch, CURLOPT_HTTPHEADER, [
			'Content-Type: application/json',
			'Content-Length: ' . strlen($data_string)
		] );
		$response = curl_exec( $ch );

		curl_close( $ch );

		if ( !$response ) {
			return 'BitNinja not running, or use incorrect version!';
		}

		return $response;
	}

	public function admin_content() {
		include_once('index.html');
        $curl = new Wp_Http_Curl();
		if ( isset( $_GET['login'] ) && $_GET['login'] == 1 ) {
			if ( self::$login_message ) {
				echo '<div class="bitninja-error">' . N;
				echo self::$login_message . N;
				echo '</div>' . N;
			}
			return;
		}
		if ( isset( $_GET['ack'] ) && $_GET['ack'] == 1 ) {
			$ack = 1;
			update_option( 'bitninja_helper_ack', $ack );
		}
		else {
			$ack = get_option( 'bitninja_helper_ack', 0 );
		}
		if ( !$ack ) {
			echo '<div class="bitninja-ack">' . N;
			echo __( 'Welcome in our Dojo! Are you ready to protect your website with us?', 'bitninja_domain' ) . N;
			echo '<a id="bitninja-ack-link" href="admin.php?page=bitninja_admin&ack=1">' . __( 'Let me in!', 'bitninja_domain' ) . '</a>' . N;
			echo '</div>' . N;
			return true;
		}
		$body['test'] = 'test';
		$result = $curl->request( self::$agent_url, array( 'body' => $body, 'method' => 'POST', 'user-agent' => 'plugin-tester', 'stream' => false, 'filename' => false, 'decompress' => false ) );

		if ( is_wp_error( $result ) || !isset( $result['body'] ) || $result['body'] != 'Error 500: Internal Server Error' ) {
			echo '<div id="bitninja_check_error">' . N;
			echo __( 'Sorry, something seems to be missing :(', 'bitninja_domain' ) . N;
			echo '</div>' . N;
			return;
		}
		echo '<div id="bitninja_check">' . N;
		echo '<div class="btn-container"><a href="admin.php?page=bitninja_admin&login=1" target="_blank"><button class="show-me">' . __( 'Show me!', 'bitninja_domain' ) . '</button></a></div>';
		echo '</div>' . N;
	}
		
}

$WP_BitNinja_Admin = WP_BitNinja_Admin::getInstance();

// end
